@@ -9,19 +9,19 @@ from __future__ import annotations
 import logging
 from abc import ABC
 from functools import partial, update_wrapper
-from typing import TYPE_CHECKING, Any, Optional, Callable, Union, TypeVar, NoReturn, Literal, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Literal, NoReturn, Optional, TypeVar, Union
 
-from ..exceptions import ParameterDefinitionError, BadArgument, CommandDefinitionError, ParamUsageError, ParserExit
+from ..exceptions import BadArgument, CommandDefinitionError, ParameterDefinitionError, ParamUsageError, ParserExit
 from ..inputs import normalize_input_type
 from ..nargs import Nargs, NargsValue
 from ..typing import T_co, TypeFunc
 from ..utils import _NotSet, str_to_bool
-from .actions import Store, StoreConst, Append, AppendConst, Count
-from .base import BaseOption, AllowLeadingDashProperty
+from .actions import Append, AppendConst, Count, Store, StoreConst
+from .base import AllowLeadingDashProperty, BaseOption
 from .option_strings import TriFlagOptionStrings
 
 if TYPE_CHECKING:
-    from ..typing import Bool, ChoicesType, InputTypeFunc, CommandCls, CommandObj, OptStr, LeadingDash, CommandMethod
+    from ..typing import Bool, ChoicesType, CommandCls, CommandMethod, CommandObj, InputTypeFunc, LeadingDash, OptStr
 
 __all__ = [
     'Option',
@@ -188,13 +188,13 @@ class Flag(BaseOption[Union[TD, TC]], actions=(StoreConst, AppendConst)):
     def register_default_cb(self, method):
         raise ParameterDefinitionError(f'{self.__class__.__name__}s do not support default callback methods')
 
-    def get_env_const(self, value: str, env_var: str) -> Tuple[Union[TC, TD], bool]:
+    def get_env_const(self, value: str, env_var: str) -> tuple[Union[TC, TD], bool]:
         try:
             parsed = self.type(value)
         except Exception as e:
-            raise ParamUsageError(self, f'unable to parse {value=} from {env_var=}: {e}') from e
+            raise ParamUsageError(self, f'unable to parse {value=} from env var={env_var!r}: {e}') from e
         if self.use_env_value and parsed != self.const and parsed != self.default:
-            raise BadArgument(self, f'invalid value={parsed!r} from {env_var=}')
+            raise BadArgument(self, f'invalid value={parsed!r} from env var={env_var!r}')
         return parsed, self.use_env_value
 
 
@@ -241,12 +241,12 @@ class TriFlag(BaseOption[Union[TD, TC, TA]], ABC, actions=(StoreConst, AppendCon
     option_strs: TriFlagOptionStrings
     alt_help: OptStr = None
     default: TD
-    consts: Tuple[TC, TA]
+    consts: tuple[TC, TA]
 
     def __init__(
         self,
         *option_strs: str,
-        consts: Tuple[TC, TA] = (True, False),
+        consts: tuple[TC, TA] = (True, False),
         alt_prefix: str = None,
         alt_long: str = None,
         alt_short: str = None,
@@ -304,14 +304,14 @@ class TriFlag(BaseOption[Union[TD, TC, TA]], ABC, actions=(StoreConst, AppendCon
         else:
             return self.consts[0]
 
-    def get_env_const(self, value: str, env_var: str) -> Tuple[Union[TC, TA, TD], bool]:
+    def get_env_const(self, value: str, env_var: str) -> tuple[Union[TC, TA, TD], bool]:
         try:
             parsed = self.type(value)
         except Exception as e:
-            raise ParamUsageError(self, f'unable to parse {value=} from {env_var=}: {e}') from e
+            raise ParamUsageError(self, f'unable to parse {value=} from env var={env_var!r}: {e}') from e
         if self.use_env_value:
             if parsed not in self.consts and parsed != self.default:
-                raise BadArgument(self, f'invalid value={parsed!r} from {env_var=}')
+                raise BadArgument(self, f'invalid value={parsed!r} from env var={env_var!r}')
             return parsed, True
         else:
             const = self.consts[0] if parsed else self.consts[1]
@@ -407,15 +407,9 @@ class ActionFlag(Flag, repr_attrs=('order', 'before_main')):
         # Allow the method to be called, regardless of whether it was specified
         if command is None:
             return self
+        # Note: If func is None, then CommandParameters._process_action_flags raises ParameterDefinitionError
         return partial(self.func, command)  # imitates a bound method
 
-    def result(self, command: CommandObj | None = None) -> Optional[Callable]:
-        if self.result_value(command):
-            if self.func:
-                return self.func
-            raise ParameterDefinitionError(f'No function was registered for {self}')
-        return None
-
 
 #: Alias for :class:`ActionFlag`
 action_flag = ActionFlag  # pylint: disable=C0103
@@ -497,14 +491,17 @@ class Counter(BaseOption[int], actions=(Count,)):
             self.default_cb = None
         return super().register_default_cb(method)
 
-    def prepare_value(self, value: Optional[str], short_combo: bool = False, pre_action: bool = False) -> int:
+    def prepare_value(self, value: Optional[str], short_combo: bool = False, env_var: str = None) -> int:
         try:
             return self.type(value)
         except (ValueError, TypeError) as e:
             combinable = self.option_strs.combinable
             if short_combo and combinable and all(c in combinable for c in value):
                 return len(value) + 1  # +1 for the -short that preceded this value
-            raise BadArgument(self, f'bad counter {value=}') from e
+            suffix = f' from env var={env_var!r}' if env_var else ''
+            raise BadArgument(self, f'bad counter {value=}{suffix}') from e
+
+    prepare_validation_value = prepare_value
 
     def validate(self, value: Any, joined: Bool = False):
         if value is None or isinstance(value, self.type):
